/**
 * 専用ハードを鳴らす
 * @author Nobuyuki TSUCHIMURA
 *
 */

public class Hard extends Thread {
    final static short VENDOR_ID = 0x1352;
    final static short PRODUCT_ID = 0x0121;
    private UsbHidCommunication usb = null;
    private byte[] buffer;
    private int bufferLen;

    private int times;
    private int msec;
    
    public Hard() { this(1,1); }
    public Hard(int times, int msec) {
        this.times = times;
        this.msec  = msec;
        try {
            usb = new UsbHidCommunication(false);
            usb.SetVendorID(VENDOR_ID);
            usb.SetProductID(PRODUCT_ID);
        } catch (UnsatisfiedLinkError e) {
        } catch (NoClassDefFoundError e) {
            //e.printStackTrace();
        }
    }
    
    public boolean isReady() {
        if (usb == null) return false;
        if (!usb.getHIDHandle()) return false;
        usb.CloseHIDDevice();
        return true;
    }
    
    @Override
    public void run() {
//      System.out.println("Hard Start");
        if (usb == null) return;
        if (!usb.getHIDHandle()) return;
        
        bufferLen = usb.GetInputReportLength();
        buffer = new byte[bufferLen];
//      System.out.println("bufferLength = " + bufferLen);
        
        try {
        	for (int i=0; i<times; i++) {
        		beepOn();
        		sleep(msec * 2 / 3);
        		beepOff();
        		sleep(msec / 3);
        	}
        } catch (InterruptedException e) {
			e.printStackTrace();
		}
        	
        usb.CloseHIDDevice();
//      System.out.println("Hard End");
    }

    public void beepOn() {
    	send(255,0); // J1 全ビットON
    }
    public void beepOff() {
    	send(0,0); // J1 全ビットOFF
    }
    
    /** コマンド送信 */
    private void send(int j1, int j2) {
    	buffer[0] = 0;  // report ID
    	buffer[1] = 0x20;
    	buffer[2] = 0x01;
    	buffer[3] = (byte)j1;
    	buffer[4] = 0x02;
    	buffer[5] = (byte)j2;
    	buffer[64] = 0;
//    	int r = 
    	usb.writeFile(buffer);
//    	System.out.println("writeFile: " + r + " bytes");
    }
}
