import java.io.*;

import org.eclipse.jface.preference.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

/**
 * @author tutimura
 *
 */
public class Preference extends Thread {
    public AlertUSB alertUSB;

    /* listener がリアルタイムに書き換える変数 */
    private int noticeSoundLen;
    private int noticeSoundCyc;
    private int noticeAudioVol;
    /* 時々表示内容を変える */
    private Label netAddrExtract;  
    private Label netAccessResult;
    
    private Group gDialog, gBeep, gHard;

    public class NoticeSoundPage extends FieldEditorPreferencePage {
        
        private NoticeSoundPage() {
                super("AlertUSB - 一般通知", GRID);
        }
        /* 共通化した */
        private Group createNoticeGroup(String text, String psname,
                SelectionListener sl) {
            Group g = new Group(getFieldEditorParent(), SWT.SHADOW_IN);
            g.setText(text);

            addField(new BooleanFieldEditor(psname, "通知する", g));

            Button b = new Button(g, SWT.PUSH);
            b.setText("テスト");
            b.addSelectionListener(sl);

            new Label(g, SWT.NONE);

            GridData data = new GridData(GridData.FILL_HORIZONTAL);
            g.setLayoutData(data);
            return g;
        }
        
        protected void createFieldEditors() {
            gDialog = createNoticeGroup("ダイヤログ", "notice.dialog", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertDialog(AlertUSB.DEV_USB);
                }
            });
            
            gBeep = createNoticeGroup("BEEP回路", "notice.beep", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertBeep(AlertUSB.DEV_USB);
                }
            });
            {
                Label l = (Label) gBeep.getChildren()[2];
                if (new Beep().isReady()) {
                    l.setText("ドライバあり");
                } else {
                    l.setText("ドライバなし");
                    gBeep.getChildren()[1].setEnabled(false);
                }
            }

            gHard = createNoticeGroup("専用ハード", "notice.hard", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertHard(AlertUSB.DEV_USB);
                }
            });
            {
                Label l = (Label) gHard.getChildren()[2];
                if (new Hard().isReady()) {
                    l.setText("ハード接続");
                } else {
                    l.setText("ハードなし");
                    gHard.getChildren()[1].setEnabled(false);
                }
            }
            
            Group g = new Group(getFieldEditorParent(), SWT.SHADOW_IN);
            g.setText("BEEP/専用ハード/sin波共通");
            GridData data = new GridData(GridData.FILL_HORIZONTAL);
            data.horizontalSpan = 3;
            g.setLayoutData(data);

            final ScaleFieldEditor lenEditor = new ScaleFieldEditor(
                    "notice.sound.len",
                    String.format("発音時間 %4d(ms) ", noticeSoundLen),
                    g, 0, 5000, 50, 500);
            final Scale lenScale = lenEditor.getScaleControl();
            lenScale.addSelectionListener(new SelectionAdapter(){
                public void widgetSelected(SelectionEvent e){
                    noticeSoundLen = lenScale.getSelection();
                    lenEditor.setLabelText(String.format("発音時間 %4d(ms)", noticeSoundLen));
                }
            });
            addField(lenEditor);

            final ScaleFieldEditor cycEditor = new ScaleFieldEditor(
            		"notice.sound.cyc",
            		String.format("断続周期 %4d(ms)", noticeSoundCyc),
            		g, 10, 1000, 5, 100);
            final Scale cycScale = cycEditor.getScaleControl();
            cycScale.addSelectionListener(new SelectionAdapter(){
            	public void widgetSelected(SelectionEvent e){
            		noticeSoundCyc = cycScale.getSelection();
            		cycEditor.setLabelText(String.format("断続周期 %4d(ms)", noticeSoundCyc));
            	}
            });
            addField(cycEditor);
        }
        @Override
        protected Control createContents(Composite parent) {
            Composite c = (Composite) super.createContents(parent);
            GridLayout layout = new GridLayout();
            layout.numColumns = 3;
            c.setLayout(layout);
            return c;
        }
    }

    public class NoticeAudioPage extends FieldEditorPreferencePage {
        private NoticeAudioPage() {
                super("AlertUSB - オーディオ", GRID);
        }
        protected void createFieldEditors() {
            addField(new BooleanFieldEditor("notice.audio",
                    "通知する", getFieldEditorParent()));
            
            String[][] sourceList = Audio.getMixerNames(true);
        	String[][] sourceNameAndVal = new String[1+sourceList.length][2];
        	sourceNameAndVal[0][0] = "既定";
        	sourceNameAndVal[0][1] = "";
        	for (int i=0; i<sourceList.length; i++) {
        		sourceNameAndVal[i+1][0] = sourceList[i][0];
        		sourceNameAndVal[i+1][1] = sourceList[i][0];
        	}

            String[][] targetList = Audio.getMixerNames(false);
        	String[][] targetNameAndVal = new String[1+targetList.length][2];
        	targetNameAndVal[0][0] = "既定";
        	targetNameAndVal[0][1] = "";
        	for (int i=0; i<targetList.length; i++) {
        		targetNameAndVal[i+1][0] = targetList[i][0];
        		targetNameAndVal[i+1][1] = targetList[i][0];
        	}

        	final ComboFieldEditor devEditor = new ComboFieldEditor(
        			"notice.audio.dev",
        			"再生デバイス",
        			sourceNameAndVal, getFieldEditorParent());
        	addField(devEditor);
        	final ComboFieldEditor mixEditor = new ComboFieldEditor(
        			"notice.audio.mix",
        			"ボリュームコントロールデバイス",
        			targetNameAndVal, getFieldEditorParent());
        	addField(mixEditor);

        	final ScaleFieldEditor volEditor = new ScaleFieldEditor(
        			"notice.audio.vol",
        			"ボリューム  " + noticeAudioVol + " ",
        			getFieldEditorParent(), 0, 100, 1, 1);
        	final Scale volScale = volEditor.getScaleControl();
        	volScale.addSelectionListener(new SelectionAdapter(){
        		public void widgetSelected(SelectionEvent e){
        			noticeAudioVol = volScale.getSelection();
        			volEditor.setLabelText("ボリューム  " + noticeAudioVol);
        		}
        	});
        	addField(volEditor);

        	createFileGroup("USBメモリの警告",      "notice.audio.usb", devEditor, mixEditor, AlertUSB.DEV_USB);
        	createFileGroup("DVD/CDメディアの警告", "notice.audio.dvd", devEditor, mixEditor, AlertUSB.DEV_DVD);
        }
        /* 共通化した */
        private void createFileGroup(String text, String psname,
        		final ComboFieldEditor devEditor,
        		final ComboFieldEditor mixEditor, int device) {
        	  String[] ext = { "*.wav", "*.*" };
            Group g = new Group(getFieldEditorParent(), SWT.SHADOW_IN);
            g.setText(text);

            final FileFieldEditor ff = new FileFieldEditor(psname, "音声ファイル", g);
            ff.setFileExtensions(ext);
            addField(ff);

            //new Label(g, SWT.NONE);
            Button b = new Button(g, SWT.PUSH);
            b.setText("テスト");
            b.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		devEditor.store(); // psに書き込むが、ファイルには保存しない
            		mixEditor.store(); // psに書き込むが、ファイルには保存しない
            		ff.store();
                    alertUSB.alertAudio(device);
                }});
            new Label(g, SWT.NONE).setText("ファイルを省略するとsin波になります");
            new Label(g, SWT.NONE);

            GridData data = new GridData(GridData.FILL_HORIZONTAL);
            data.horizontalSpan = 3;
            g.setLayoutData(data);
        }
    }

    public class NoticeNetPage extends FieldEditorPreferencePage {
        private NoticeNetPage() {
                super("AlertUSB - ネット通知", GRID);
        }
        protected void createFieldEditors() {
            addField(new BooleanFieldEditor("notice.net",
                    "通知する", getFieldEditorParent()));
            final ComboFieldEditor netProto = new ComboFieldEditor("notice.net.proto",
                    "プロトコル", new String[][] {
                    {"http", "http://"},
                    {"https", "https://"},
            }, getFieldEditorParent());
            addField(netProto);
            
            final StringFieldEditor netAddr = new StringFieldEditor("notice.net.addr",
                    "アドレス", getFieldEditorParent());
            addField(netAddr);

            Button eval = new Button(getFieldEditorParent(), SWT.PUSH);
            netAddrExtract = new Label(getFieldEditorParent(), SWT.BORDER);
            Button access = new Button(getFieldEditorParent(), SWT.PUSH);
            netAccessResult = new Label(getFieldEditorParent(), SWT.BORDER);
            
            eval.setText("環境変数展開");
            access.setText("テスト");

            eval.addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                	netProto.store();
                	netAddr.store();
                	netAddrExtract.setText(alertUSB.getNetURL(AlertUSB.DEV_USB));
                	netAddrExtract.pack();
                }
            });
            access.addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                	netProto.store();
                	netAddr.store();
                	netAddrExtract.setText(alertUSB.getNetURL(AlertUSB.DEV_USB));
                	netAddrExtract.pack();
        			netAccessResult.setText("通信中");
        			netAccessResult.pack();
                	alertUSB.alertNet(AlertUSB.DEV_USB,  new ResultListener() {
                    	@Override
                    	public void finished(boolean success, Object data) {
                    		System.out.println(success);
                    		if (success) {
                    			netAccessResult.setText("成功");
                    		} else {
                    			netAccessResult.setText("失敗" + (String)data);
                    		}
                    		netAccessResult.pack();
                    	}
                    });
                }
            });

            {
                Text t = new Text(getFieldEditorParent(), SWT.MULTI);
                t.setEditable(false);
                t.setText("\n" +
                        "アドレスには bat と同じ環境変数が使えます。\n" +
                        "例：\n" +
                        "%USERNAME% →ユーザ名\n" +
                        "%COMPUTERNAME% →ホスト名\n" +
                        "\n" +
                        "見つけたメディアをあらわす特殊な変数もあります。\n" +
                        "%FIND_DEV% →:USB:DVD:\n" +
                        "%FIND_DRV% →:D:E:Q:\n" +
                "");
                GridData data = new GridData(GridData.FILL_HORIZONTAL);
                data.horizontalSpan = 3;
                t.setLayoutData(data);
                t.pack();
            }
        }
    }

    public class WatchPage extends FieldEditorPreferencePage {
    	private Label usb, dvd;
        private WatchPage() {
                super("AlertUSB - 監視デバイス", GRID);
        }
        protected void createFieldEditors() {
            addField(new BooleanFieldEditor("watch.usb",
                    "USBメモリ", getFieldEditorParent()));
            addField(new BooleanFieldEditor("watch.dvd",
                    "CD/DVDドライブ", getFieldEditorParent()));

            new Label(getFieldEditorParent(), SWT.CENTER); // 隙間を作る
            Label mesg = new Label(getFieldEditorParent(), SWT.CENTER);
            mesg.setText("デバイスの存在するドライブレターを列挙します。");
            
            Button b = new Button(getFieldEditorParent(), SWT.PUSH);
            b.setText("更新");
            b.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		usb.setText("USB: " + new DeviceList().getAllDevices(AlertUSB.DEV_USB));
            		dvd.setText("DVD: " + new DeviceList().getAllDevices(AlertUSB.DEV_DVD));
            		
            	}
            });
            usb = new Label(getFieldEditorParent(), SWT.LEFT);
            dvd = new Label(getFieldEditorParent(), SWT.LEFT);
    		usb.setText("USB: 更新ボタンを押してください");
    		dvd.setText("DVD: 更新ボタンを押してください");
        }
    }

    public class AboutPage extends FieldEditorPreferencePage {
        private AboutPage() {
                super("AlertUSB - about", GRID);
        }
        protected void createFieldEditors() {
            Text t = new Text(getFieldEditorParent(), SWT.MULTI);
            t.setEditable(false);
            t.setText(
                    "Version " + AlertUSB.VERSION + "\n" +
                    "https://cs.kwansei.ac.jp/~tutimura/AlertUSB/\n" +
                    "\n" +
                    "(C) Noyubuki Tsuchimura, \n" + 
                    "Phillip Gibbons (inpout32.dll), \n" +
                    "Xavier Clerc (wave sound BEEP), \n" +
                    "KSL Embedded (USB HID access), \n" +
                    "Timothy Wall (JNA), \n" +
                    "Eclipse Foundation Software (SWT)\n" +
                    "\n" +
                    AlertUSB.USAGE_MESSAGE +
            "");
            t.pack();
        }
    }

    private PreferenceStore ps;
    public static final boolean isUnix = File.separatorChar == '/';
    private Shell shell;
    private PreferenceDialog dialog;
    
    public Preference(AlertUSB alertUSB, Shell shell) {
        this.alertUSB = alertUSB;
        this.shell = shell;

        String path = System.getProperty("java.class.path");
        if (!path.contains(File.pathSeparator)
                && path.endsWith(".jar")) { // jar による起動
            // jar と同じフォルダの rc
            path = path.replaceFirst("jar$", "rc");
        } else {
            path = "AlertUSB.rc";  // カレントの rc
        }
        System.out.println("config file : " + path);
        ps = new PreferenceStore(path);
        ps.setDefault("notice.dialog", true);
        ps.setDefault("notice.audio", true);
        ps.setDefault("notice.beep", true);
        ps.setDefault("notice.sound.len", 2500);
        ps.setDefault("notice.sound.cyc", 100);
        ps.setDefault("notice.net.proto", "http://");
        ps.setDefault("watch.usb", true);
        if (isUnix) setDefaultUnix();
        else        setDefaultWin();
    }
   
    /** 初期値の上書き(UNIX) */
    private void setDefaultUnix() {
        ps.setDefault("notice.beep", false);
    }
    
    /** 初期値の上書き(Windows) */
    private void setDefaultWin() {
    }
    
    public static boolean isUnix()       { return isUnix; }

    public boolean noticeDialog() { return ps.getBoolean("notice.dialog"); }
    public boolean noticeAudio()  { return ps.getBoolean("notice.audio"); }
    public String noticeAudioDev(){ return ps.getString("notice.audio.dev"); }
    public String noticeAudioMix(){ return ps.getString("notice.audio.mix"); }
    public int noticeAudioVol()   { return noticeAudioVol; }
    public String noticeAudioUSB(){ return ps.getString("notice.audio.usb"); }
    public String noticeAudioDVD(){ return ps.getString("notice.audio.dvd"); }
    public boolean noticeBeep()   { return ps.getBoolean("notice.beep"); }
    public boolean noticeHard()   { return ps.getBoolean("notice.hard"); }
    public int noticeSoundLen()   { return noticeSoundLen; }
    public int noticeSoundCyc()   { return noticeSoundCyc; }
    public boolean noticeNet()    { return ps.getBoolean("notice.net"); }
    public String noticeNetProto() { return ps.getString("notice.net.proto"); }
    public String noticeNetAddr()  { return ps.getString("notice.net.addr"); }
    
    public boolean watchUSB()     { return ps.getBoolean("watch.usb"); }
    public boolean watchDVD()     { return ps.getBoolean("watch.dvd"); }

    public boolean load() {
        boolean ret = true;
        try {
            ps.load();
        } catch (FileNotFoundException e) {
            ret = false;
        } catch (IOException e) {
            e.printStackTrace();
            ret = false;
        }
        noticeSoundLen = ps.getInt("notice.sound.len");
        noticeSoundCyc = ps.getInt("notice.sound.cyc");
        noticeAudioVol = ps.getInt("notice.audio.vol");
        return ret;
    }
    
    public void open() {
        PreferenceManager manager = new PreferenceManager();

        PreferenceNode noticeSoundNode = new PreferenceNode("notice-sound", new NoticeSoundPage());
        manager.addToRoot(noticeSoundNode);
        
        PreferenceNode noticeAudioNode = new PreferenceNode("notice-net", new NoticeAudioPage());
        manager.addToRoot(noticeAudioNode);
        
        PreferenceNode noticeNetNode = new PreferenceNode("notice-net", new NoticeNetPage());
        manager.addToRoot(noticeNetNode);
        
        PreferenceNode watchNode = new PreferenceNode("watch", new WatchPage());
        manager.addToRoot(watchNode);

        PreferenceNode aboutNode = new PreferenceNode("about", new AboutPage());
        manager.addToRoot(aboutNode);

        dialog = new PreferenceDialog(shell, manager);
        dialog.setPreferenceStore(ps);
        //shell.getDisplay().asyncExec(new Update()); // call run()
        dialog.open(); // && wait
    }

    class Update extends Thread {
        public void run() {
            boolean hard = true;
            
            while (!shell.isDisposed()) {
                System.out.println("running...");
                Label lHard = (Label) gHard.getChildren()[2];
                if (!isUnix() && new Hard().isReady()) {
                    if (!hard) {
                        lHard.setText("ハード接続");
                        gHard.getChildren()[1].setEnabled(true);
                        gHard.redraw();
                        hard = true;
                    }
                } else {
                    if (hard) {
                        lHard.setText("ハードなし");
                        gHard.getChildren()[1].setEnabled(false);
                        gHard.redraw();
                        hard = false;
                    }
                }
                
                try {
                    sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                //if (!shell.isDisposed()) dialog.updateButtons();
            }
            System.out.println("running... stopped");
        }
    }
}
