import javax.sound.sampled.*;

// https://codeday.me/jp/qa/20190312/377934.html
// https://docs.oracle.com/javase/jp/1.5.0/guide/sound/programmer_guide/chapter2.html#112445

public class SoundAudit {
	public static void main(String[] args) {
		try {
			System.out.println("OS: " + System.getProperty("os.name") + " "
					+ System.getProperty("os.version") + "/"
					+ System.getProperty("os.arch") + "\n"
					+ "Java: "
					+ System.getProperty("java.version")
					+ " (" + System.getProperty("java.vendor") + ")\n");
			
			int no = 0;
			for (Mixer.Info thisMixerInfo : AudioSystem.getMixerInfo()) {
				System.out.println("Mixer "+(no++)+": " + thisMixerInfo.getDescription()
				+ " [" + Audio.deMessyCode(thisMixerInfo.getName()) + "]");
				Mixer thisMixer = AudioSystem.getMixer(thisMixerInfo);
				
				System.out.println("自身 " + thisMixer.getControls().length);
				System.out.println("入力 " + thisMixer.getSourceLineInfo().length);
				for (Line.Info thisLineInfo : thisMixer.getSourceLineInfo()) {
					//if (thisLineInfo.getLineClass().getName().equals("javax.sound.sampled.Port")) {
						Line thisLine = thisMixer.getLine(thisLineInfo);
						if (!(thisLine instanceof Clip)) {
						thisLine.open();
						System.out.println("  Source Port: " + Audio.deMessyCode(thisLineInfo.toString()));
						for (Control thisControl : thisLine.getControls()) {
							System.out.println(AnalyzeControl(thisControl));
						}
						thisLine.close();
					} else {
						System.out.println("  unknown Line " + thisLineInfo);
					}
				}
				
				System.out.println("出力 " + thisMixer.getTargetLineInfo().length);
				for (Line.Info thisLineInfo : thisMixer.getTargetLineInfo()) {
					if (thisLineInfo.getLineClass().getName().equals("javax.sound.sampled.Port")) {
						Line thisLine = thisMixer.getLine(thisLineInfo);
						thisLine.open();
						System.out.println("  Target Port: " + Audio.deMessyCode(thisLineInfo.toString()));
						//System.out.println("  isSource() = " + ((Port.Info)thisLineInfo).isSource());
						for (Control thisControl : thisLine.getControls()) {
							System.out.println(AnalyzeControl(thisControl));
						}
						thisLine.close();
					} else {
						System.out.println("  unknown Line " + thisLineInfo);
					}
				}
				System.out.println();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static String AnalyzeControl(Control thisControl) {
		String type = Audio.deMessyCode(thisControl.getType().toString());
		if (thisControl instanceof BooleanControl) {
			return "    Control: " + type + " (boolean)";
		}
		if (thisControl instanceof CompoundControl) {
			System.out.println("    Control: " + type + " (compound - values below)");
			String toReturn = "";
			for (Control children : ((CompoundControl) thisControl).getMemberControls()) {
				toReturn += "  " + AnalyzeControl(children) + "\n";
			}
			return toReturn.substring(0, toReturn.length() - 1);
		}
		if (thisControl instanceof EnumControl) {
			return "    Control:" + type + " (enum: " + thisControl.toString() + ")";
		}
		if (thisControl instanceof FloatControl) {
			return "    Control: " + type + " (float: from " + ((FloatControl) thisControl).getMinimum() + " to "
					+ ((FloatControl) thisControl).getMaximum() + ")";
		}
		return "    Control: unknown type";
	}
}
